@@ -1,3 +1,14 @@
+/* 
+**	dbadmin.c	-
+**
+**
+** Copyright (c) 1996 James E. Harrell, Jr.
+**
+** This software is provided "as is" without any expressed or implied warranty.
+**
+**
+*/
+
 #include <stdio.h>
 #include "cgic.h"
 #include "msql.h"
@@ -12,26 +23,19 @@ int  name_length;
 int       state;
 int       action;
 int       numfields;
-int       delitemnum;
 char      *fieldnames;
 int       sock;
-char      newdb[MAX_DATABASENAME_LEN];
 char      field[MAX_FIELDS][MAX_FIELDNAME_LEN];
 char      type[MAX_FIELDS][5];
 char      lengthStr[MAX_FIELDS][5];
 char      notnul[MAX_FIELDS][2];
 char      key[MAX_FIELDS][2];
-char      textquery[MAX_TEXTQUERY_LEN];
-char      searchby[MAX_FIELDNAME_LEN];
-char      searchdata[MAX_ITEM_LEN];
 char      operation[5];
 m_result  *res;
 m_result  *res2;
 m_row     cur;
 m_field   *curField;
 int       dberror;
-char      dbname[MAX_DATABASENAME_LEN];
-char      tablename[MAX_TABLENAME_LEN];
 char      qbuf[MAX_TEXTQUERY_LEN];
 cgiFormResultType cgiError;
 
@@ -40,6 +44,7 @@ void giveDebugInfo(void);
 int  getState(void);
 void giveTitle(void);
 void connectMSQL(void);
+void getNewDBName(void);
 void connectDB(void);
 void connectTable(void);
 void doTitlePage(void);
@@ -180,12 +185,31 @@ void connectMSQL(void)
   }
 }
 
+void getNewDBName(void)
+{
+#ifdef DEBUG
+  fprintf(cgiOut,"<BR>Getting space requirements for database name.\n<BR>\n");
+#endif
+ if ((cgiError=cgiFormStringSpaceNeeded("thedb",&name_length))==cgiFormNotFound)
+    { 
+    fprintf(cgiOut,"You have requested to create a database without\n");
+    fprintf(cgiOut,"specifying a database name!\n");
+    giveButtonBar();
+    exit(1);
+    }
+  #ifdef DEBUG
+  fprintf(cgiOut,"<BR>Currently allocating %d bytes of memory.<BR>\n",name_length);
+  #endif
+  thedb = (char *)malloc(sizeof(char)*name_length);
+  cgiError=cgiFormString("thedb",thedb,name_length);
+}
+
 void connectDB(void)
 {
 #ifdef DEBUG
   fprintf(cgiOut,"<BR>Getting space requirements for database name.\n<BR>\n");
 #endif
-  if ((cgiError=cgiFormStringSpaceNeeded("dbname",&name_length))==cgiFormNotFound)
+ if ((cgiError=cgiFormStringSpaceNeeded("thedb",&name_length))==cgiFormNotFound)
     { 
     fprintf(cgiOut,"You have requested to connect to a database without\n");
     fprintf(cgiOut,"specifying a database name!\n");
@@ -196,32 +220,32 @@ void connectDB(void)
   fprintf(cgiOut,"<BR>Currently allocating %d bytes of memory.<BR>\n",name_length);
   #endif
   thedb = (char *)malloc(sizeof(char)*name_length);
-  cgiError=cgiFormString("dbname",thedb,name_length);
+  cgiError=cgiFormString("thedb",thedb,name_length);
   if(msqlSelectDB(sock,thedb) < 0)
     {
-    fprintf(cgiOut,"ERROR selecting database %s<BR>\n",dbname);
+    fprintf(cgiOut,"ERROR selecting database %s<BR>\n",thedb);
     fprintf(cgiOut,"\n%s<BR>\n\n",msqlErrMsg);
     msqlClose(sock);
     giveButtonBar();
     exit(1);
     }
   #ifdef DEBUG
-  fprintf(cgiOut,"<BR>Connectd to database %s.\n<BR>",thedb);
+  fprintf(cgiOut,"<BR>Connected to database %s.\n<BR>",thedb);
   #endif
 }
 
 void connectTable(void)
 {
-  if ((cgiError=cgiFormStringSpaceNeeded("tablename",&table_name_length)) ==
+  if ((cgiError=cgiFormStringSpaceNeeded("thetable",&table_name_length)) ==
     cgiFormNotFound)
     {
-    fprintf(cgiOut,"You have requested to connect to a table in database %s\n",dbname);
+    fprintf(cgiOut,"You have requested to connect to a table in database %s\n",thedb);
     fprintf(cgiOut,"without specifying the table!\n");
     giveButtonBar();
     exit(1);
     }
   thetable = (char *)malloc(sizeof(char)*table_name_length);
-  cgiError=cgiFormString("tablename",thetable,table_name_length);
+  cgiError=cgiFormString("thetable",thetable,table_name_length);
   #ifdef DEBUG
   fprintf(cgiOut,"<BR>All queries currently apply to table %s.<BR>\n",thetable);
   #endif
@@ -318,7 +342,7 @@ void addDatabase(void)
   fprintf(cgiOut,"<CENTER><TABLE BORDER=0>\n");
   fprintf(cgiOut,"<TR WIDTH=85%%>\n");
   fprintf(cgiOut,"<TD ALIGN=CENTER WIDTH=33%%>New Database Name:</TD>\n");
-  fprintf(cgiOut,"<TD ALIGN=CENTER WIDTH=33%%><INPUT NAME=\"newdb\" SIZE=%d></TD>\n",MAX_DATABASENAME_LEN);
+  fprintf(cgiOut,"<TD ALIGN=CENTER WIDTH=33%%><INPUT NAME=\"thedb\" SIZE=25></TD>\n");
   fprintf(cgiOut,"<TD ALIGN=CENTER WIDTH=33%%><INPUT TYPE=submit VALUE=\"Add Database\"></TD>\n");
   fprintf(cgiOut,"</TR></TABLE></CENTER>\n");
 }
@@ -379,7 +403,7 @@ int x=0;
       { 
       case NORMAL: fprintf(cgiOut,"<LI> %s<BR>\n",cur[0]);
            break;
-      case RADIO_LIST: fprintf(cgiOut,"<DD><INPUT TYPE=RADIO NAME=\"dbname\" VALUE=%s",cur[0]);
+      case RADIO_LIST: fprintf(cgiOut,"<DD><INPUT TYPE=RADIO NAME=\"thedb\" VALUE=\"%s\"",cur[0]);
                        if (x==1) fprintf(cgiOut," CHECKED");
                        fprintf(cgiOut,"> %s<BR>\n",cur[0]);
                        
@@ -397,10 +421,9 @@ void doAddDB(void)
 {
   giveTitle();
   connectMSQL();
-  cgiError=cgiFormString("newdb",newdb,MAX_DATABASENAME_LEN);
-  if (cgiError==cgiFormSuccess)
-    {
-    if(msqlCreateDB(sock,newdb) < 0)
+  getNewDBName();
+
+  if(msqlCreateDB(sock,thedb) < 0)
       {
       fprintf(cgiOut,"\n<BR>mSQL Command Failed!\n<BR>Server Error = %s\n<BR>\n",msqlErrMsg);
       msqlClose(sock);
@@ -408,15 +431,8 @@ void doAddDB(void)
       exit(1);
       }
      else
-      { fprintf(cgiOut,"<BR>Database \"%s\" created.<BR>\n",newdb); }
-    /* showDatabases(NORMAL); */
-    addDatabase();
-    }
-   else
-    {
-    fprintf(cgiOut,"\n<BR>An invalid name was entered for the database!\n<BR>\n");
-    fprintf(cgiOut,"Please check that the name is correct and reload page.\n");
-    }
+      { fprintf(cgiOut,"<BR>Database \"%s\" created.<BR>\n",thedb); }
+  addDatabase();  
   msqlClose(sock);
 }
 
@@ -428,22 +444,18 @@ void doRemoveDB(void)
 {
   giveTitle();
   connectMSQL();
-  cgiError=cgiFormString("dbname",dbname,MAX_DATABASENAME_LEN);
-  if (cgiError==cgiFormSuccess)
-    {
-    if(msqlDropDB(sock,dbname) < 0)
-      {
-      fprintf(cgiOut,"\nmSQL Command failed!\n<BR>Server error = %s\n<BR>\n",msqlErrMsg);
-      msqlClose(sock);
-      giveButtonBar();
-      exit(1);
-      }
-     else
-      { fprintf(cgiOut,"Database \"%s\" dropped!\n<BR>\n",dbname); }
-    removeDatabase();
+  connectDB();
+
+  if(msqlDropDB(sock,thedb) < 0)
+    {     
+    fprintf(cgiOut,"\nmSQL Command failed!\n<BR>Server error = %s\n<BR>\n",msqlErrMsg);
+    msqlClose(sock);
+    giveButtonBar();
+    exit(1);
     }
    else
-    { fprintf(cgiOut,"Invalid database selected for removal!\n<BR>\n"); }
+    { fprintf(cgiOut,"Database \"%s\" dropped!\n<BR>\n",thedb); }
+  removeDatabase();
   msqlClose(sock);
 }
 
@@ -454,38 +466,34 @@ void doModifyStructure(void)
 {
   giveTitle();
   connectMSQL();
-  cgiError=cgiFormString("dbname",dbname,MAX_DATABASENAME_LEN);
-  if (cgiError==cgiFormSuccess)
+  connectDB();
+
+  if (msqlSelectDB(sock,thedb) < 0)
     {
-    if (msqlSelectDB(sock,dbname) < 0)
-      {
-      fprintf(cgiOut,"ERROR selecting database %s\n<BR>\n",dbname);
-      fprintf(cgiOut,"\n%s<BR>\n\n",msqlErrMsg);
-      msqlClose(sock);
-      giveButtonBar();
-      exit(1);
-      }
-     else
-      {
-/*    if TABLESEXIST */
-      showCurrentTables();
-      giveAddTable();
-      }
+    fprintf(cgiOut,"ERROR selecting database %s\n<BR>\n",thedb);
+    fprintf(cgiOut,"\n%s<BR>\n\n",msqlErrMsg);
+    msqlClose(sock);
+    giveButtonBar();
+    exit(1);
     }
    else
-    { fprintf(cgiOut,"Invalid database selected for modify!\n<BR>\n"); }
+    {
+/*  if TABLESEXIST */
+    showCurrentTables();
+    giveAddTable();
+    }
   msqlClose(sock);
 }
 
 void showCurrentTables(void)
 {
-  fprintf(cgiOut,"<H3>The following table definitions exist for database: %s</H3>",dbname);
+  fprintf(cgiOut,"<H3>The following table definitions exist for database: %s</H3>",thedb);
   fprintf(cgiOut,"<HR>\n");
   res = msqlListTables(sock);
   if (!res)
     {
-    fprintf(cgiOut,"\nERROR : Unable to list tables in database %s<BR>\n",dbname);
-      giveButtonBar();
+   fprintf(cgiOut,"\nERROR : Unable to list tables in database %s<BR>\n",thedb);
+    giveButtonBar();
     exit(1);
     }
    else
@@ -500,7 +508,7 @@ void showCurrentTables(void)
         res2 = msqlListFields(sock,cur[0]);
         if (!res2)
           {
-          fprintf(cgiOut,"ERROR : Couldn't find %s in %s!<BR>\n",cur[0],dbname);
+          fprintf(cgiOut,"ERROR : Couldn't find %s in %s!<BR>\n",cur[0],thedb);
           msqlFreeResult(res);
       giveButtonBar();
           exit(1);
@@ -540,8 +548,8 @@ void showCurrentTables(void)
             }
           fprintf(cgiOut,"</TABLE><BR>\n");
           fprintf(cgiOut,"<FORM METHOD=POST ACTION=%s/dbadmin>\n",RELPATH);
-          fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"dbname\" VALUE=\"%s\">\n",dbname);
-          fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"tablename\" VALUE=%s>\n",cur[0]);
+          fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"thedb\" VALUE=\"%s\">\n",thedb);
+          fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"thetable\" VALUE=%s>\n",cur[0]);
           fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"state\" VALUE=5>\n");
           fprintf(cgiOut,"<INPUT TYPE=SUBMIT VALUE=\"REMOVE TABLE\">\n");
           fprintf(cgiOut,"</FORM>\n");
@@ -561,7 +569,7 @@ void giveAddTable(void)
   fprintf(cgiOut,"<H3>Add table definition with:</H3>\n");
   fprintf(cgiOut,"<FORM METHOD=POST ACTION=%s/dbadmin>\n",RELPATH);
   fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"state\" VALUE=6>\n");
-  fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"dbname\" VALUE=\"%s\">\n",dbname);
+  fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"thedb\" VALUE=\"%s\">\n",thedb);
   fprintf(cgiOut,"Number of fields = ");
   fprintf(cgiOut,"<INPUT NAME=\"numfields\" SIZE=2>\n");
   fprintf(cgiOut,"<INPUT TYPE=SUBMIT VALUE=\"CREATE TABLE...\">\n");
@@ -575,44 +583,28 @@ void doRemoveTable(void)
 {
   giveTitle();
   connectMSQL();
-  cgiError=cgiFormString("dbname",dbname,MAX_DATABASENAME_LEN);
-  if (cgiError==cgiFormSuccess) 
+  connectDB();
+  connectTable(); 
+
+  #ifdef DEBUG
+  fprintf(cgiOut,"\n<BR>Table %s being removed from %s\n",thetable,thedb);
+  #endif
+  strcpy(qbuf,"DROP TABLE ");
+  strcat(qbuf,thetable);
+  #ifdef DEBUG
+  fprintf(cgiOut,"Query = %s<BR>\n",qbuf);
+  #endif
+  if (msqlQuery(sock,qbuf) == -1)
     {
-    cgiError=cgiFormString("tablename",tablename,MAX_TABLENAME_LEN);
-    if (cgiError==cgiFormSuccess)
-      {
-      if (msqlSelectDB(sock,dbname) < 0)
-        {
-        fprintf(cgiOut,"ERROR selecting database %s\n<BR>\n",dbname);
-        fprintf(cgiOut,"\n%s<BR>\n\n",msqlErrMsg);
-        msqlClose(sock);
-      giveButtonBar();
-        exit(1);
-        }
-     else
-        {
-        fprintf(cgiOut,"\n<BR>Table %s removed from %s,\n",tablename,dbname);
-        strcpy(qbuf,"DROP TABLE ");
-        strcat(qbuf,tablename);
-#ifdef DEBUG
-        fprintf(cgiOut,"Query = %s<BR>\n",qbuf);
-#endif
-        if (msqlQuery(sock,qbuf) == -1)
-          {
-          fprintf(cgiOut,"ERROR: unable to drop table %s from %s.<BR>\n",tablename,dbname);
-          fprintf(cgiOut,"Invalid query = %s<BR>\n",qbuf);
-          msqlClose(sock);
-      giveButtonBar();
-          exit(1);
-          }
-        }
-      }
-     else
-      { fprintf(cgiOut,"\nERROR : Table %s not found!<BR>\n",tablename); }
+    fprintf(cgiOut,"ERROR: unable to drop table %s from %s.<BR>\n",thetable,thedb);
+    fprintf(cgiOut,"Invalid query = %s<BR>\n",qbuf);
+    msqlClose(sock);
+    giveButtonBar();
+    exit(1);
     }
    else
-    { fprintf(cgiOut,"\nERROR : Database %s not found!<BR>\n",dbname); }
-  
+    { fprintf(cgiOut,"\n<BR>Table %s removed from %s.\n",thetable,thedb); }
+    
   showCurrentTables();
   giveAddTable();
   msqlClose(sock);
@@ -627,9 +619,8 @@ int x;
 
   giveTitle();
   connectMSQL();
-  cgiError=cgiFormString("dbname",dbname,MAX_DATABASENAME_LEN);
-  if (cgiError==cgiFormSuccess) 
-    {
+  connectDB();
+ 
     cgiError=cgiFormIntegerBounded("numfields",&numfields,1,MAX_FIELDS,0);
     if (cgiError==cgiFormSuccess)
       { 
@@ -637,10 +628,10 @@ int x;
       fprintf(cgiOut,"<CENTER>\n");
       fprintf(cgiOut,"<FORM METHOD=POST ACTION=%s/dbadmin>\n",RELPATH);
       fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=state VALUE=7>\n");
-      fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"dbname\" VALUE=\"%s\">\n",dbname);
+      fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"thedb\" VALUE=\"%s\">\n",thedb);
       fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"numfields\" VALUE=%d>\n",numfields);
       fprintf(cgiOut,"<TABLE BORDER=0><TR><TH>Table Name:</TH></TR>\n");
-      fprintf(cgiOut,"<TR><TD><INPUT NAME=\"tablename\" VALUE=\"(new_table_name)\" SIZE=%d></TD></TR></TABLE>\n",MAX_TABLENAME_LEN);
+      fprintf(cgiOut,"<TR><TD><INPUT NAME=\"thetable\" VALUE=\"(new_table_name)\" SIZE=25></TD></TR></TABLE>\n");
       fprintf(cgiOut,"<TABLE CELLPADDING=3 BORDER=4>\n");
       fprintf(cgiOut,"<TR>\n");
       fprintf(cgiOut,"<TH WIDTH=15%%>FIELD\n");
@@ -701,14 +692,6 @@ int x;
       giveButtonBar();
       exit(1);
       }
-    }
-   else
-    {
-    fprintf(cgiOut,"ERROR: Unable to connect to database!<BR>\n");
-    msqlClose(sock);
-      giveButtonBar();
-    exit(1);
-    }
   msqlClose(sock);
 }
 
@@ -721,24 +704,14 @@ int x;
 
   giveTitle();
   connectMSQL();
+  connectDB();
+  connectTable();
 
-  cgiFormString("dbname",dbname,MAX_DATABASENAME_LEN);
-  cgiFormString("tablename",tablename,MAX_TABLENAME_LEN);
   cgiFormInteger("numfields",&numfields,0);
   getFields();
-#ifdef DEBUG
+  #ifdef DEBUG
   fprintf(cgiOut,"NUMBER OF FIELDS = %d<BR>\n",numfields);
-#endif
-  if(msqlSelectDB(sock,dbname) < 0)
-    {
-    fprintf(cgiOut,"ERROR selecting database %s<BR>\n",dbname);
-    fprintf(cgiOut,"\n%s<BR>\n\n",msqlErrMsg);
-    msqlClose(sock);
-      giveButtonBar();
-    exit(1);
-    }
-#ifdef DEBUG
-  fprintf(cgiOut,"CREATE TABLE %s (<BR>\n",tablename);
+  fprintf(cgiOut,"CREATE TABLE %s (<BR>\n",thetable);
   for(x=1; x<=numfields; x++)
     {
     fprintf(cgiOut,"%s %s",&field[x-1][0],&type[x-1][0]); 
@@ -748,9 +721,9 @@ int x;
     if (x!=numfields) { fprintf(cgiOut,",<BR>\n"); }
      else { fprintf(cgiOut," )<BR>\n"); }
     }
-#endif
+  #endif
   strcpy(qbuf,"CREATE TABLE ");
-  strcat(qbuf,tablename);
+  strcat(qbuf,thetable);
   strcat(qbuf," ( ");
   for(x=1; x<=numfields; x++)
     {
@@ -774,13 +747,13 @@ int x;
 
   if (msqlQuery(sock,qbuf) == -1)
     {
-    fprintf(cgiOut,"ERROR: unable to add table %s to %s.<BR>\n",tablename,dbname);
+   fprintf(cgiOut,"ERROR: unable to add table %s to %s.<BR>\n",thetable,thedb);
     fprintf(cgiOut,"Invalid query = %s<BR>\n",qbuf);
     msqlClose(sock);
       giveButtonBar();
     exit(1);
     }
-  fprintf(cgiOut,"Table %s added.\n",tablename);
+  fprintf(cgiOut,"Table %s added.\n",thetable);
   showCurrentTables();
   giveAddTable();
   msqlClose(sock);
@@ -927,15 +900,7 @@ void doTableSelection(void)
 {
   giveTitle();
   connectMSQL();
-  cgiFormString("dbname",dbname,MAX_DATABASENAME_LEN);
-  if(msqlSelectDB(sock,dbname) < 0)
-    {
-    fprintf(cgiOut,"ERROR selecting database %s<BR>\n",dbname);
-    fprintf(cgiOut,"\n%s<BR>\n\n",msqlErrMsg);
-    msqlClose(sock);
-      giveButtonBar();
-    exit(1);
-    }
+  connectDB();
   giveSearch(); 
   giveInsert();
   giveTextQuery();
@@ -964,14 +929,11 @@ void giveInsert(void)
 
 void showTables(void)
 {
-/*  fprintf(cgiOut,"<H3>Select a table to insert data:</H3>\n"); */
-/*  fprintf(cgiOut,"<FORM METHOD=POST ACTION=%s/dbadmin>\n",RELPATH); */
-/*  fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=state VALUE=10>\n"); */
 int x=0;
   res = msqlListTables(sock);
   if (!res)
     {
-    fprintf(cgiOut,"\nERROR : Unable to list tables in database %s<BR>\n",dbname);
+   fprintf(cgiOut,"\nERROR : Unable to list tables in database %s<BR>\n",thedb);
     msqlClose(sock);
       giveButtonBar();
     exit(1);
@@ -979,11 +941,11 @@ int x=0;
   while(cur = msqlFetchRow(res))
     { 
     x++;
-    fprintf(cgiOut,"<DD><INPUT TYPE=RADIO NAME=\"tablename\" VALUE=\"%s\" ",cur[0]);
+    fprintf(cgiOut,"<DD><INPUT TYPE=RADIO NAME=\"thetable\" VALUE=\"%s\" ",cur[0]);
     if (x==1) { fprintf(cgiOut," CHECKED"); }
     fprintf(cgiOut,"> %s\n<BR>",cur[0]);
     }
-  fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"dbname\" VALUE=\"%s\">\n",dbname);
+  fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"thedb\" VALUE=\"%s\">\n",thedb);
 /*  fprintf(cgiOut,"<INPUT TYPE=SUBMIT VALUE=\"New Record... \">\n"); */
 /*  fprintf(cgiOut,"</FORM><HR>\n"); */
 }
@@ -993,13 +955,13 @@ void giveCannedSelect(void)
   fprintf(cgiOut,"<H3>Select a query:</H3>\n");
   fprintf(cgiOut,"<FORM METHOD=POST ACTION=%s/dbadmin>",RELPATH);
   fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"state\" VALUE=xx>\n");
-  fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"dbname\" VALUE=\"%s\">\n",dbname);
+  fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"thedb\" VALUE=\"%s\">\n",thedb);
   fprintf(cgiOut,"SELECT...<BR>\n");
   fprintf(cgiOut,"<DD><INPUT TYPE=CHECKBOX NAME=\"columns\"> * <BR>\n");
   fprintf(cgiOut,"FROM...<BR>\n");
   res = msqlListTables(sock);
   while((cur = msqlFetchRow(res)))
-    { fprintf(cgiOut,"<DD><INPUT TYPE=CHECKBOX NAME=\"tablename\" VALUE=\"%s\"> %s\n<BR>",cur[0],cur[0]); }
+    { fprintf(cgiOut,"<DD><INPUT TYPE=CHECKBOX NAME=\"thetable\" VALUE=\"%s\"> %s\n<BR>",cur[0],cur[0]); }
   fprintf(cgiOut,"</FORM>\n");
   fprintf(cgiOut,"<HR>\n");
 }
@@ -1009,9 +971,9 @@ void giveTextQuery(void)
   fprintf(cgiOut,"<H3>Enter a mSQL query by hand if you prefer:</H3>\n");
   fprintf(cgiOut,"<FORM METHOD=POST ACTION=%s/dbadmin>",RELPATH);
   fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"state\" VALUE=9>\n");
-  fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"dbname\" VALUE=\"%s\">\n",dbname);
+  fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"thedb\" VALUE=\"%s\">\n",thedb);
   fprintf(cgiOut,"<TEXTAREA COLS=60 ROWS=10 NAME=\"textquery\">\n");
-  fprintf(cgiOut,"SELECT * FROM (tablename)\n");
+  fprintf(cgiOut,"SELECT * FROM (table_name)\n");
   fprintf(cgiOut,"</TEXTAREA>\n");
   fprintf(cgiOut,"<INPUT TYPE=SUBMIT VALUE=\"Submit mSQL query...\">\n");
   fprintf(cgiOut,"</FORM>\n");
@@ -1037,9 +999,6 @@ char *command;
   thetextquery=(char *)malloc(sizeof(char)*query_len);
   cgiFormString("textquery",thetextquery,query_len);
 
-/*
-  cgiFormString("textquery",textquery,MAX_TEXTQUERY_LEN);
-*/
   if (msqlQuery(sock,thetextquery) == -1)
     {
     fprintf(cgiOut,"ERROR: unable to perform query on database %s.<BR>\n",thedb);
@@ -1049,15 +1008,9 @@ char *command;
     giveButtonBar();
     exit(1);
     }
-/*
-  thetextquery = (char *)malloc(MAX_TEXTQUERY_LEN*sizeof(char));
-  bzero(thetextquery,MAX_TEXTQUERY_LEN);
-  thetextquery = strcpy(thetextquery,textquery); 
-  command = (char *)malloc(100*sizeof(char));
-  bzero(command,100);
-*/
+
   command = strtok(thetextquery," ");
-  if (strcmp(command,"SELECT")==0)
+  if ((strcmp(command,"SELECT")==0) || (strcmp(command,"select")==0))
     { 
     fprintf(cgiOut,"<CENTER>The query \"%s\" returned the following values:\n",thetextquery);
     showSelectData();
@@ -1128,8 +1081,8 @@ int size=1;
     fprintf(cgiOut,"<CENTER>\n");
     fprintf(cgiOut,"<FORM METHOD=POST ACTION=%s/dbadmin>\n",RELPATH);
     fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"state\" VALUE=11>\n");
-    fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"dbname\" VALUE=\"%s\">\n",thedb);
-    fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"tablename\" VALUE=\"%s\">\n",thetable);
+    fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"thedb\" VALUE=\"%s\">\n",thedb);
+    fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"thetable\" VALUE=\"%s\">\n",thetable);
     fprintf(cgiOut,"<TABLE WIDTH=75%% BORDER=2 CELLPADDING=4>\n");
     while((curField = msqlFetchField(res)))
       {
@@ -1164,7 +1117,7 @@ void giveBackQueryPage(void)
   fprintf(cgiOut,"<CENTER><HR>\n");
   fprintf(cgiOut,"<FORM METHOD=POST ACTION=%s/dbadmin>\n",RELPATH);
   fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"state\" VALUE=8>\n");
-  fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"dbname\" VALUE=\"%s\">\n",thedb);
+  fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"thedb\" VALUE=\"%s\">\n",thedb);
   fprintf(cgiOut,"<INPUT TYPE=SUBMIT VALUE=\"RETURN TO MAIN QUERY PAGE\">\n");
   fprintf(cgiOut,"</FORM>\n");
   fprintf(cgiOut,"<HR></CENTER>\n");
@@ -1177,7 +1130,7 @@ void addCannedQuery(void)
 {
 int count=0;
 int x,lengthstr;
-char temp[MAX_ITEM_LEN];
+char *the_data;
 
   giveTitle();
 
@@ -1210,23 +1163,29 @@ char temp[MAX_ITEM_LEN];
   while(curField = msqlFetchField(res))
     {
     cgiFormStringSpaceNeeded(curField->name,&lengthstr);
+    the_data = (char *)malloc(sizeof(char)*lengthstr);
+    cgiFormString(curField->name,the_data,lengthstr);
+
     if (count>0) {strcat(qbuf,", ");}
     count++;
     switch (curField->type)
       {
       case INT_TYPE:
-                     cgiFormString(curField->name,temp,lengthstr);
-                     strcat(qbuf,temp);
+      case REAL_TYPE:
+                     if (lengthstr > 1)
+                       { strcat(qbuf,the_data); }
+                      else
+                       { strcat(qbuf,"NULL"); }
                      break;
-      case CHAR_TYPE: 
-                     strcat(qbuf,"'");
-                     cgiFormString(curField->name,temp,lengthstr);
-                     strcat(qbuf,temp);
-                     strcat(qbuf,"'");
-                     break;
-      case REAL_TYPE: 
-                     cgiFormString(curField->name,temp,lengthstr);
-                     strcat(qbuf,temp);
+      case CHAR_TYPE:
+                     if (lengthstr > 1)
+                       { 
+                       strcat(qbuf,"'");
+                       strcat(qbuf,the_data);
+                       strcat(qbuf,"'");
+                       }
+                      else                       
+                       { strcat(qbuf,"NULL"); }
                      break;
       default:
                      fprintf(cgiOut,"Error: Unknown mSQL field type");
@@ -1234,6 +1193,7 @@ char temp[MAX_ITEM_LEN];
                      exit(1);
                      break;
       }
+    free(the_data);
     }
   strcat(qbuf,")");
   #ifdef DEBUG
@@ -1243,7 +1203,7 @@ char temp[MAX_ITEM_LEN];
   
   if (msqlQuery(sock,qbuf) == -1)
     {
-    fprintf(cgiOut,"ERROR: unable to perform query on database %s.<BR>\n",dbname);
+   fprintf(cgiOut,"ERROR: unable to perform query on database %s.<BR>\n",thedb);
     fprintf(cgiOut,"Invalid query = %s<BR>\n",qbuf);
     giveInsertForm();
     msqlClose(sock);
@@ -1280,8 +1240,8 @@ void giveSearchForm(void)
   fprintf(cgiOut,"<H3>Search of table %s:</H3>\n",thetable);
   fprintf(cgiOut,"<CENTER>\n");
   fprintf(cgiOut,"<FORM METHOD=POST ACTION=%s/dbadmin>\n",RELPATH);
-  fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"dbname\" VALUE=\"%s\">\n",thedb);
-  fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"tablename\" VALUE=%s>\n",thetable);
+  fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"thedb\" VALUE=\"%s\">\n",thedb);
+  fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"thetable\" VALUE=%s>\n",thetable);
   fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"state\" VALUE=14>\n");
   fprintf(cgiOut,"<TABLE WIDTH=75%% BORDER=2 CELLPADDING=4>\n");
   res = msqlListFields(sock,thetable);
@@ -1450,18 +1410,34 @@ char *rec_id;
       strcat(rec_id," = ");
       if (curField->type==CHAR_TYPE)
         {
-        rec_id_len += 1;
-        rec_id=(char *)realloc(rec_id,rec_id_len);
-        strcat(rec_id,"'");
+        if (cur[z] != NULL)
+          {
+          rec_id_len += 1;
+          rec_id=(char *)realloc(rec_id,rec_id_len);
+          strcat(rec_id,"'");
+          }
         }
+/* HERE */
+      if (cur[z] == NULL)
+        {
+        rec_id_len += 4;
+        rec_id=(char *)realloc(rec_id,rec_id_len);
+        strcat(rec_id,"NULL");
+        }
+       else
+        {
       rec_id_len += strlen(cur[z]);
       rec_id=(char *)realloc(rec_id,rec_id_len);
       strcat(rec_id,cur[z]);
+        }
       if (curField->type==CHAR_TYPE)
         {
-        rec_id_len += 1;
-        rec_id=(char *)realloc(rec_id,rec_id_len);
-        strcat(rec_id,"'");
+        if (cur[z] != NULL)
+          {
+          rec_id_len += 1;
+          rec_id=(char *)realloc(rec_id,rec_id_len);
+          strcat(rec_id,"'");
+          }
         }
       z++;
       }
@@ -1476,8 +1452,8 @@ char *rec_id;
       fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"state\" VALUE=16>\n"); 
       fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"action\" VALUE=\"UPDATE\">\n");
       }
-    fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"dbname\" VALUE=\"%s\">\n",thedb);
-    fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"tablename\" VALUE=\"%s\">\n",thetable);
+    fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"thedb\" VALUE=\"%s\">\n",thedb);
+    fprintf(cgiOut,"<INPUT TYPE=HIDDEN NAME=\"thetable\" VALUE=\"%s\">\n",thetable);
     res2 = msqlListFields(sock,thetable);
     y=0;
     while (curField = msqlFetchField(res2))
@@ -1494,11 +1470,12 @@ char *rec_id;
     fprintf(cgiOut,"</FORM>\n");
     /* fprintf(cgiOut,"<BR>\n"); */
     }
+/*
   fprintf(cgiOut,"</CENTER><HR>\n");
-  if (z>0) { free(rec_id); }
+  if (z>0) { free(rec_id); } 
   free(action);
-  msqlFreeResult(res);
-  
+  msqlFreeResult(res); 
+*/
 }
 
 
@@ -1575,6 +1552,7 @@ int z;
 char *field_value;
 int  field_value_len;
 
+
   query_len = 8;
   query=(char *)malloc(sizeof(int)*query_len);
   strcpy(query,"UPDATE ");
@@ -1603,26 +1581,46 @@ int  field_value_len;
       rec_new=(char *)realloc(rec_new,rec_new_len);
       strcat(rec_new,curField->name);
       strcat(rec_new," = ");
-      if (curField->type==CHAR_TYPE)
-        {
-        rec_new_len += 1;
-        rec_new=(char *)realloc(rec_new,rec_new_len);
-        strcat(rec_new,"'");
-        }
 
       cgiFormStringSpaceNeeded(curField->name,&field_value_len);
       field_value=(char *)malloc(sizeof(char)*field_value_len);
       cgiFormString(curField->name,field_value,field_value_len);
-      rec_new_len += field_value_len;
-      rec_new=(char *)realloc(rec_new,rec_new_len);
-      strcat(rec_new,field_value);
-      free(field_value);
       
-      if (curField->type==CHAR_TYPE)
+      if (strcmp(field_value,"(null)")==0) 
         {
-        rec_new_len += 1;
+        rec_new_len += 4;
         rec_new=(char *)realloc(rec_new,rec_new_len);
-        strcat(rec_new,"'");
+        strcat(rec_new,"NULL");
+        }
+       else
+        {      
+        
+
+        if (curField->type==CHAR_TYPE)
+          {
+          rec_new_len += 1;
+          rec_new=(char *)realloc(rec_new,rec_new_len);
+          strcat(rec_new,"'");
+          }
+
+/*
+      cgiFormStringSpaceNeeded(curField->name,&field_value_len);
+      field_value=(char *)malloc(sizeof(char)*field_value_len);
+      cgiFormString(curField->name,field_value,field_value_len);
+*/ 
+
+
+        rec_new_len += field_value_len;
+        rec_new=(char *)realloc(rec_new,rec_new_len);
+        strcat(rec_new,field_value);
+        free(field_value);
+      
+        if (curField->type==CHAR_TYPE)
+          {
+          rec_new_len += 1;
+          rec_new=(char *)realloc(rec_new,rec_new_len);
+          strcat(rec_new,"'");
+          }
         }
       z++;
       }
